'''A POP client class.

Originally by Konrad Hinsen <hinsen@ibs.ibs.fr>
This version by Andy Dustman <andy@CCMSD.chem.uga.edu>
Modified 1996/12/05

Attention: We have never seen an official POP specification, and
we have tested this code on two machines only. Use at your own risk.

Example (all addresses and data are fictitious!):

>>> from poplib import POP
>>> pop = POP('pop-server.python.org')  # connect to the server
>>> pop.identify('jones', '**top-secret**')  # login with user/password
>>> pop.status()  # return number of messages and total length
(42, 199426)
>>> len(pop)  # number of messages
42
>>> print pop.retrieve(0)  # get the first message as list of strings
['From: John Cleese <cleese@python.org>\012',
'To: Terry Jones <jones@python.org>\012','\012',
'Just testing this terrific POP client!\012','\012','--john\012']
>>> print pop[0]  # returns string containing message
From: John Cleese <cleese@python.org>
To: Terry Jones <jones@python.org>

Just testing this terrific POP client!

--john
>>> pop.delete(0)  # delete a message from the server
>>> pop.close()  # end session
>>> # alternate method for connecting to server
>>> pop = POP('pop-server.python.org','jones','**top-secret**')
>>> pop.connect()  # identifies with stored name/password
>>> # move all messages to a local mailbox
>>> mailbox = open('mail/semprini','a')
>>> for i in range(len(pop)): pop.retrieve(i,mailbox); del pop[i]
>>>
% # does same as above every 900 seconds (15 minutes)
% python poplib.py 900 pop-server.python.org jones '**top-secret**' >>mail/semprini &
[993]
% 
'''

import socket, string
from time import time, localtime, strftime

POPError = 'POPError'

class POP:

    def __init__(self, server, user = None, password = None,
		 port = 110, debug = 0):
	self.debug = debug
	self.server = server
	self.user = user
	self.password = password
	self.port = port
	self.buf = ""
	if not user: self.connect()

    def connect(self):
	try:
	    self.sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
	    self.sock.connect(self.server, self.port)
	    self._checkResponse()
	    if self.user: self.identify(self.user, self.password)
	except socket.error, err:
	    raise POPError, (socket.error, err)
	

    def _sendline(self, line):
	if self.debug: print 'CLIENT: ' + line
	try:
	    self.sock.send(line + '\r\n')
	except socket.error, err:
	    raise POPError, (socket.error, err)

    def _getline(self):
	line = ''
	data, self.buf = self.buf, ''
	while 1:
	    i = string.find(data, '\r\n')
	    if i >= 0: break
	    try:
		data = data + self.sock.recv(500)
	    except socket.error, err:
		raise POPerror, (socket.error, err)
	line, self.buf = data[:i], data[i+2:]
	if self.debug: print 'SERVER: ' + line
	return line

    def _getlines(self):
	lines = []
	while 1:
	    line = self._getline()
	    if line == '.': break
	    lines.append(line+'\n')
	return lines

    def _checkResponse(self):
	self.response = self._getline()
	if self.response[:3] == '+OK': return
	if self.response[:4] != '-ERR':
	    raise POPError, 'Unexpected response'
	else:
	    raise POPError, self.response[5:]

    def rexec(self, cmd):
	self._sendline(cmd)
	self._checkResponse()

    def identify(self, user, password = None):
	self.rexec('USER ' + user)
	if password is not None:
	    self.rexec('PASS ' + password)

    def status(self):
	self.rexec('STAT')
	response = string.split(self.response)
	return string.atoi(response[1]), string.atoi(response[2])

    def __len__(self):
	l,b = self.status()
	return l

    def list(self, message = None):
	if message is not None:
	    self.rexec('LIST ' + `message+1`)
	    response = string.split(self.response)
	    return string.atoi(response[1]), string.atoi(response[2])
	else:
	    self.rexec('LIST')
	    lines = self._getlines()
	    return map(lambda s: 
		       tuple(map(string.atoi, string.split(s))), lines)

    def retrieve(self, message, outfile=None, lines=0):
	"""retrieve(message, outfile=None, lines=0)
	where:
	   message: message number (first message is 0)
	   outfile: optional file to store messages in
	   lines: how many lines to retrieve (0 means all)

        If outfile is a string, a file is created by that name in
	write mode and the message is written to that file. Else if
	outfile is a file object (or something with a write method),
	it is written to that file in mailbox format, i.e., there is
	a leading From line, and the last line of the message must
	be blank."""
	if not lines: self.rexec('RETR ' + `message+1`)
	else: self.rexec('TOP ' + `message+1` + ' ' + `lines`)
	if not outfile: return self._getlines()
	if type(outfile) == type(''):
	    out = open(outfile,'w')
	else:
	    out = outfile
	    # maintain mailbox format with appropriate From line
	    out.write(strftime('From poplib  %c\n',localtime(time())))
	last = ''
	while 1:
	    line = self._getline()
	    if line == '.': break
	    out.write(line+'\n')
	    last = line
	# end of message must be a blank line for mailbox format
	if type(outfile) == type(''):
	    	    out.close()
	elif last != '': out.write('\n')

    def __getitem__(self, key):
	try:
	    s = self.retrieve(key)
	except POPError:
	    raise IndexError, "Could not retrieve message"
	return string.joinfields(s,'')

    def delete(self, message):
	self.rexec('DELE ' + `message+1`)

    def __delitem__(self, key):
	self.delete(key)

    def close(self):
	try:
	    self.rexec('QUIT')
	    self.sock.close()
	    del self.sock
	except POPError: pass
	except socket.error: pass

if __name__ == "__main__":
    # primitive mail POPping daemon
    import sys
    from time import sleep

    delay = eval(sys.argv[1])
    p = POP(sys.argv[2],sys.argv[3],sys.argv[4])
    while 1:
	p.connect()
	for i in range(len(p)):
	    p.retrieve(i, sys.stdout)
	    p.delete(i)
	p.close()
	sleep(delay)
